/*****************************************************************************************  
 *  OpenSourceSaverView.m
 *  OpenSourceSaver
 *
 *  Created by Doug Diego on Thu Jun 13 2002.
 *  Copyright (c) 2002 Diego.org. All rights reserved.
 *
 *  
 *  This software is distributed under the terms of the GNU General Public License (GPL)
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation.  No representations are made about the suitability of this
 * software for any purpose.  It is provided "as is" without express or 
 * implied warranty.
 *
 * THANKS for all the feed back that I have recieved.  It has been very helpful.
 ******************************************************************************************/

#import "OpenSourceSaverView.h"

static NSString *DOSS = @"org.diego.OpenSourceSaver";

@implementation OpenSourceSaverView


- (id)initWithFrame:(NSRect)frame isPreview:(BOOL)isPreview
{
    NSArray *files;
    int x;
    self = [super initWithFrame:frame isPreview:isPreview];
    if (self) {
        /* init data */
        i = 0;
        previousLocation = 0;
    
        /* load user defined prefs */
        [self loadPrefs];
    
        /* set font color/size */
        [self prepareAttributes];
        
        /* Now we will either get the source code for this program to
         * display or the source code from some directory */
        if(displayOwnSource){
            //NSLog(@"Loading own source file into array");
            /* add source files to array */
            arrayOfFiles =[[NSMutableArray alloc ] init ];
            [arrayOfFiles addObject:@"OpenSourceSaverView.m.txt"];
            [arrayOfFiles addObject:@"OpenSourceSaverView.h.txt"];
            [arrayOfFiles addObject:@"OpenSourceSaverPrefs.m.txt"];
            [arrayOfFiles addObject:@"OpenSourceSaverPrefs.h.txt"];
            sourceCount = [arrayOfFiles count];
        }else{
            //NSLog(@"Loading source files from some directory into array");
            
            /* This call recursively grabs all files from a directory and puts in an arrary */
            files = [[NSFileManager defaultManager] subpathsAtPath:theSearchRoot];
    
            /* I will need to remove some items from the array.  So am 
             * creating  a new NSMutable Array and copying all the values in to it */
            arrayOfFiles =[[NSMutableArray alloc ] init ];
            for (x=0; x< [files count]; x++) {
                [arrayOfFiles insertObject:[files objectAtIndex:x] atIndex:x];
            }
            sourceCount = [arrayOfFiles count];
          
            /* Since I can't scroll through binary files, I need to
             * filter out everything that is not displayable */
            [self filterSource];
        }

        //NSLog(@"%@", [arrayOfFiles description]);
        
        /* initilize first source to display */
        currentSource = 0;
        
        /* load the fist doc */
        [self loadDoc];
        
        /* set animation speed */
        [self setAnimationTimeInterval:1/speed];
    }
    return self;
}

- (void)filterSource
{
    int x;
    NSString *name;
    NSString *extension;

    /* Go through each file and look at the extenstion.  If it is an know extenstion,
     * then it will leave it, if not it will remove it from the array */
    for (x=0; x< [arrayOfFiles count]; ) {
        name= [[NSString alloc] initWithString:[arrayOfFiles objectAtIndex:x]];
        //NSLog(@"%i examing %@", x,[theSearchRoot stringByAppendingString:name]);
        
        extension = [[[theSearchRoot stringByAppendingString:name] pathExtension] lowercaseString];
        //NSLog(@"Name %@ Extenstion %@", name, extension );

        /* TODO: make this configurable in the preferences, let user choose file types */
        if ([@"txt" caseInsensitiveCompare:extension] == NSOrderedSame){
            x++;
        }else if([@"m" caseInsensitiveCompare:extension] == NSOrderedSame){
            x++;
        }else if([@"h" caseInsensitiveCompare:extension] == NSOrderedSame){
            x++;
        }else if([@"c" caseInsensitiveCompare:extension] == NSOrderedSame){
            x++;
        }else if([@"cpp" caseInsensitiveCompare:extension] == NSOrderedSame){
            x++;
        }else if([@"java" caseInsensitiveCompare:extension] == NSOrderedSame){
            x++;
        }else if([@"pl" caseInsensitiveCompare:extension] == NSOrderedSame){
            x++;
        }else if([@"js" caseInsensitiveCompare:extension] == NSOrderedSame){
            x++;
        }else if([@"css" caseInsensitiveCompare:extension] == NSOrderedSame){
            x++;
        }else if([@"html" caseInsensitiveCompare:extension] == NSOrderedSame){
            x++;
        }else if([@"htm" caseInsensitiveCompare:extension] == NSOrderedSame){
            x++;
        }else if([@"tex" caseInsensitiveCompare:extension] == NSOrderedSame){
            x++;
        }else if([@"xml" caseInsensitiveCompare:extension] == NSOrderedSame){
            x++;
        }else if([@"strings" caseInsensitiveCompare:extension] == NSOrderedSame){
            x++;
        }else if([@"plist" caseInsensitiveCompare:extension] == NSOrderedSame){
            x++;
        }else if([@"cgi" caseInsensitiveCompare:extension] == NSOrderedSame){
            x++;
        }else if([@"py" caseInsensitiveCompare:extension] == NSOrderedSame){
            x++;
        }else if([@"php" caseInsensitiveCompare:extension] == NSOrderedSame){
            x++;
        }else if([@"cfm" caseInsensitiveCompare:extension] == NSOrderedSame){
            x++;
        /* Yeah Right, only real source code can be displayed :) 
        }else if([@"cs" caseInsensitiveCompare:extension] == NSOrderedSame){
            x++;
        }else if([@"vb" caseInsensitiveCompare:extension] == NSOrderedSame){
            x++;
        */
        }else{
            [arrayOfFiles removeObjectAtIndex:x];
            if(x == 0 ){
            }else{
                x--;
            }
        }
        /* I'm sure that I missed lots of differne exentions that you might want
         * If I missed one, add it yourself.  Or wait and I'll make it a 
         * preferences some day */
         
        [name release];
    }
    /* Since we removed a bunch we need to reset the count */
    sourceCount = [arrayOfFiles count];
    
    /* Check to see what was filtered out */
    /*
    for (x=0; x< [arrayOfFiles count]; x++) {
        name = [arrayOfFiles objectAtIndex:x];
        NSLog(@"%i checking %@", x,[theSearchRoot stringByAppendingString:name]);
    }
    */

}

- (void)loadDoc
{
    NSString * fileName;
    NSBundle *thisBundle;

    theCode = [[NSMutableString alloc] init];

    /* make sure the current source exist 
    if not, start over again*/
    if(currentSource < sourceCount ){
    }else{
        currentSource = 0;
    }

    if(displayOwnSource){
        /* get the source file from the resource */
        thisBundle = [NSBundle bundleForClass:[self class]];
        fileName = [thisBundle pathForResource:[arrayOfFiles objectAtIndex:currentSource] ofType:nil];
        //NSLog(@"fileName= %@", fileName);
        text = [[NSString alloc] initWithContentsOfFile: fileName];
    }else{
        fileName = [arrayOfFiles objectAtIndex:currentSource];
        //NSLog(@"%i fileName= %@ ", currentSource, [theSearchRoot stringByAppendingString:fileName]);
        text = [[NSString alloc] initWithContentsOfFile: [theSearchRoot stringByAppendingString:fileName]];
    }
}

- (void)startAnimation
{
    [super startAnimation];
}

- (void)stopAnimation
{
    [super stopAnimation];
}

- (void)drawRect:(NSRect)rect
{
    NSRect bounds = [self bounds];
    
    /* set background color to black */
    [[NSColor blackColor] set];
    [NSBezierPath fillRect:bounds];
    
    /* Draw line of Code */
    [self drawString:theCode atPoint:point];
}

- (void)animateOneFrame
{
    NSRect bounds = [self bounds];
    NSRange range; 
    int j, x;
    int length;
    
    //NSLog(@"animateOneFrame");
    
    /* set the point to draw at the bottom of screen */
    point.x = bounds.origin.x;
    point.y = bounds.origin.y;
    
    i++;
    /* get length of source */
    length = [text length];

    x = 1;
    /* previousLocation was a \n, so increment one */
    previousLocation++;

    //NSLog(@"previousLocation %i, length %i", previousLocation, length );

    /* Check to see if we have reached the end of the doc */
    if(previousLocation>length){
        /* we have reached in of doc, so load new one */
        previousLocation = 0;
        currentSource++;
        [text release];
        [theCode release];
        [self loadDoc];
    }
    for(j = previousLocation; j < length; j++){
        //NSLog(@"j=%i uni=%i str=%@",j,[text characterAtIndex:j], [text substringWithRange:NSMakeRange(j, 5)]);
        if([text characterAtIndex:j] == 10  || [text characterAtIndex:j] == 13){
            /* Get the range of the line */
            range = NSMakeRange(previousLocation,j - previousLocation);
            /* Make sure it is not null and get substring from range */
            if([text substringWithRange:range] != nil){
                [theCode appendString:[text substringWithRange:range]];
                [theCode appendString:@"\n"];
            }
            previousLocation = j;
            break;
        }
    }     
    [self setNeedsDisplay: YES];
}

- (BOOL) hasConfigureSheet { 
    return YES;
} 

- (NSWindow*)configureSheet {
    return [[OpenSourceSaverPrefs sharedPreferences] window];
}

-(void)storePrefs {
    /* Save preferences */
    /* http://developer.apple.com/techpubs/macosx/AdditionalTechnologies/PreferencePanes/Tasks/Sample.html */
    //NSLog(@"storePrefs");
    //NSLog(@"storePrefs() - fontSize=%i", fontSize);
    
    CFPreferencesSetAppValue((CFStringRef)@"speed", [NSNumber numberWithFloat:speed], (CFStringRef)DOSS);
    CFPreferencesSetAppValue((CFStringRef)@"fontSize", [NSNumber numberWithFloat:fontSize], (CFStringRef)DOSS);
    //NSLog(@"%@", fontName);
    CFPreferencesSetAppValue((CFStringRef)@"fontName", fontName, (CFStringRef)DOSS);
    CFPreferencesSetAppValue((CFStringRef)@"searchRoot", theSearchRoot, (CFStringRef)DOSS);
    
    if ( displayOwnSource ){
        //NSLog(@"storePrefs displayOwnSource: TRUE");
        CFPreferencesSetAppValue( (CFStringRef)@"displayOwnSource", kCFBooleanTrue, (CFStringRef)DOSS );
    }else{
        //NSLog(@"storePrefs displayOwnSource: TRUE");
        CFPreferencesSetAppValue( (CFStringRef)@"displayOwnSource",kCFBooleanFalse, (CFStringRef)DOSS );
    }
    
    CFPreferencesAppSynchronize((CFStringRef)DOSS);
} 

-(void)loadPrefs {
    BOOL initPrefs = NO;
    CFPropertyListRef test;   
    NSLog(@"loadPrefs");

    test = CFPreferencesCopyAppValue((CFStringRef) @"speed", (CFStringRef) DOSS);
    speed = [(NSNumber *)test floatValue];
    if (speed < 0 || speed > 30 || test == NULL) {
        initPrefs = YES;
    }
        
    test = CFPreferencesCopyAppValue((CFStringRef) @"fontSize", (CFStringRef) DOSS);
    fontSize = [(NSNumber *)test floatValue];
    if (fontSize <8 || fontSize > 75 || test == NULL) {
        initPrefs = YES;
    }
        
    test = CFPreferencesCopyAppValue((CFStringRef) @"searchRoot", (CFStringRef) DOSS);
    theSearchRoot = (NSMutableString *)test;
    if (theSearchRoot == nil) {
        initPrefs = YES;
    }
    
    test = CFPreferencesCopyAppValue((CFStringRef) @"fontName", (CFStringRef) DOSS);
    fontName = (NSString *)test;
    NSLog(@"fontName= %@", fontName);
    if (fontName == nil) {
        NSLog(@"fontName is nill");
        initPrefs = YES;
    }
        
    test = CFPreferencesCopyAppValue((CFStringRef) @"displayOwnSource", (CFStringRef) DOSS);
    if ( test && CFGetTypeID(test) == CFBooleanGetTypeID() ) {
        if(CFBooleanGetValue(test)){
            displayOwnSource = TRUE;
            // NSLog(@"loadPrefs displayOwnSource: TRUE");
        }else{
            displayOwnSource = FALSE;
            // NSLog(@"loadPrefs displayOwnSource: False");
        }
    }else{
        initPrefs = YES;
    }
    
    
    test = CFPreferencesCopyAppValue((CFStringRef) @"fontColorRed", (CFStringRef) DOSS);
    fontColorRed = [(NSNumber *)test floatValue];
    if (test == NULL) {
        initPrefs = YES;
    }
    test = CFPreferencesCopyAppValue((CFStringRef) @"fontColorBlue", (CFStringRef) DOSS);
    fontColorBlue = [(NSNumber *)test floatValue];
    if (test == NULL) {
        initPrefs = YES;
    }
    test = CFPreferencesCopyAppValue((CFStringRef) @"fontColorGreen", (CFStringRef) DOSS);
    fontColorGreen = [(NSNumber *)test floatValue];
    if (test == NULL) {
        initPrefs = YES;
    }
    test = CFPreferencesCopyAppValue((CFStringRef) @"fontColorAlpha", (CFStringRef) DOSS);
    fontColorAlpha = [(NSNumber *)test floatValue];
    if (test == NULL) {
        initPrefs = YES;
    }
    
        
    /* If values aren't set or are out of range, recreate */
    if (initPrefs) {
        NSLog(@"initilizing");
        theSearchRoot = @"";
        displayOwnSource = TRUE;
        speed = 15.0;
        fontSize = 25.0;
        fontName = @"Times";
        fontColorRed = 0.54;
        fontColorGreen = 1.00;
        fontColorBlue = 0.20;
        fontColorAlpha = 1.00;
        [self storePrefs];
    }
    return;       
}


- (void)drawString:(NSString *)myString atPoint: (NSPoint) thePoint
{
    NSSize stringSize;
    
    stringSize = [myString sizeWithAttributes:attributes];
    [myString drawAtPoint: thePoint withAttributes:attributes]; 
}

- (void)prepareAttributes
{
    NSColor *color;
    
    color = [NSColor colorWithCalibratedRed:fontColorRed green:fontColorGreen blue:fontColorBlue alpha:fontColorAlpha ];

    attributes = [[NSMutableDictionary alloc] init];
    // NSLog(@"prepareAttributes");
    // NSLog(@"prepareAttributes fontName=%@ fontSize=%d", fontName, fontSize);
    // NSLog(@"prepareAttributes color red=%.2f green=%.2f blue=%.2f alpha=%.2f", fontColorRed, fontColorGreen, fontColorBlue, fontColorAlpha);
    /* set font and size */
    [attributes setObject: 
          [NSFont fontWithName: fontName size: fontSize] forKey: NSFontAttributeName];
    // [attributes setObject: 
    //    [NSFont fontWithName: @"Helvetica" size: fontSize] forKey: NSFontAttributeName];

    /* set font color */
    //[attributes setObject: [NSColor greenColor] forKey: NSForegroundColorAttributeName];
    [attributes setObject: color forKey: NSForegroundColorAttributeName];
}


@end
